@@ -85,8 +85,13 @@ class SyncWorkerProxy(WorkerProxy):
         # (limits and retry_configs already processed by WorkerBuilder)
         worker_cls = _create_worker_wrapper(self.worker_cls, self.limits, self.retry_configs)
 
+        # CRITICAL: Pass _from_proxy=True to bypass auto_init logic in Worker.__new__
+        # This prevents infinite recursion when the worker class has auto_init=True
+        init_kwargs = dict(self.init_kwargs)
+        init_kwargs["_from_proxy"] = True
+
         # Create the worker instance directly
-        self._worker = worker_cls(*self.init_args, **self.init_kwargs)
+        self._worker = worker_cls(*self.init_args, **init_kwargs)
 
     def __getattr__(self, name: str):
         """Intercept method calls with caching for maximum performance.
@@ -192,7 +197,10 @@ class SyncWorkerProxy(WorkerProxy):
             if self.retry_configs is not None:
                 submit_retry_config = self.retry_configs.get("submit") or self.retry_configs.get("*")
 
-            if submit_retry_config is not None and submit_retry_config.num_retries > 0:
+            # Apply retry logic if configured (num_retries > 0 or retry_until is set)
+            if submit_retry_config is not None and (
+                submit_retry_config.num_retries > 0 or submit_retry_config.retry_until is not None
+            ):
                 context = {
                     "method_name": fn.__name__ if hasattr(fn, "__name__") else "anonymous_function",
                     "worker_class_name": "TaskWorker",
